--  Create_Constraints v. 1.1
--
--  This script generates a list (or script, if spooled) of "create
--  constraint" statements, based on the primary-key, foreign-key,
--  and check constraints currently in an Oracle database. It must
--  be run under a user with DBA rights, and has been tested under
--  Oracle version 8.1.7. Note that constraints owned by SYS or
--  SYSTEM are excluded from the output, and that the output is
--  sorted within each of the three sections by OWNER and TABLE_NAME.
--
-- 
--                                          W. S. Pollack
--                                          11/2002
--                                          bill@billanddot.com
--                                          http://www.billanddot.com/downloads.htm
set verify off
set termout off
set feedback off
set echo off
set pagesize 0
set linesize 1040
set long 32767
set trims on
set space 0
set recsep off
--
column nl newline
--
select '-- ================================ --' nl,
       '-- "Create_Constraints" Version 1.1 --' nl,
       '--                                  --' nl,
       '-- ' ||
--           lpad(' ', 6-ceil(length(trim(NAME))/2)) || trim(NAME) ||
             lpad(' ',16-ceil(length(trim(NAME))/2)) || trim(NAME) ||
--           ', ' || to_char(SYSDATE,'MM/DD/YYYY @ HH24:MI') ||
--           lpad(' ', 6-floor(length(trim(NAME))/2)) ||
             lpad(' ',16-floor(length(trim(NAME))/2)) ||
                                          ' --' nl,
       '-- ================================ --' nl,
       ' ' nl
from V$DATABASE
;
select ' ' nl,
       '-- ======================= --' nl,
       '-- Primary-Key Constraints --' nl,
       '-- ======================= --' nl,
       ' ' nl
from DUAL
;
select 'alter table    ' || D1.OWNER || '.' || D1.TABLE_NAME nl,
       'add constraint ' || D1.OWNER || '.' || D1.CONSTRAINT_NAME nl,
       'primary key   ' ||
       (select '('  || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 1
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 2
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 3
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 4
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 5
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 6
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 7
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 8
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 9
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 10
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 11
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 12
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 13
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 14
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 15
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 16
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 17
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 18
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 19
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 20
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 21
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 22
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 23
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 24
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 25
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 26
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 27
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 28
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 29
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 30
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 31
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = CONSTRAINT_NAME
        and    INDEX_OWNER = OWNER
        and    COLUMN_POSITION = 32
       ) || 
       ')' nl,
       ';' nl
from   DBA_CONSTRAINTS D1,
       DBA_IND_COLUMNS D2
where  CONSTRAINT_TYPE = 'P'
and    OWNER not in ('SYS','SYSTEM')
and    OWNER = INDEX_OWNER
and    CONSTRAINT_NAME = INDEX_NAME
and    COLUMN_POSITION = 1
order  by OWNER, D1.TABLE_NAME
;
select ' ' nl,
       '-- ======================= --' nl,
       '-- Foreign-Key Constraints --' nl,
       '-- ======================= --' nl,
       ' ' nl
from DUAL
;
select 'alter table    ' || D1.OWNER || '.' || D1.TABLE_NAME nl,
       'add constraint ' || D1.OWNER || '.' || D1.CONSTRAINT_NAME nl,
       'foreign key   ' ||
       (select '('  || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 1
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 2
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 3
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 4
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 5
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 6
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 7
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 8
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 9
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 10
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 11
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 12
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 13
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 14
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 15
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 16
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 17
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 18
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 19
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 20
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 21
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 22
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 23
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 24
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 25
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 26
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 27
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 28
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 29
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 30
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 31
       ) || 
       (select ', ' || COLUMN_NAME
        from   DBA_IND_COLUMNS
        where  INDEX_NAME = R_CONSTRAINT_NAME
        and    INDEX_OWNER = R_OWNER
        and    COLUMN_POSITION = 32
       ) || 
       ')' nl,
       'references     ' || TABLE_OWNER || '.' || D2.TABLE_NAME ||
       decode(DELETE_RULE,'CASCADE',' on delete cascade','') nl,
       ';' nl
from   DBA_CONSTRAINTS D1,
       DBA_IND_COLUMNS D2
where  CONSTRAINT_TYPE = 'R'
and    OWNER not in ('SYS','SYSTEM')
and    R_OWNER = INDEX_OWNER
and    R_CONSTRAINT_NAME = INDEX_NAME
and    COLUMN_POSITION = 1
order  by OWNER, D1.TABLE_NAME
;
select ' ' nl,
       '-- ================= --' nl,
       '-- Check Constraints --' nl,
       '-- ================= --' nl,
       ' ' nl
from DUAL
;
select 'alter table    ' || OWNER || '.' || TABLE_NAME,
       'add constraint ' || OWNER || '.' || CONSTRAINT_NAME nl,
       'check' nl,
       '(' nl,
       SEARCH_CONDITION,
       ')' nl,
       ';' nl
from   DBA_CONSTRAINTS
where  CONSTRAINT_TYPE = 'C'
and    OWNER not in ('SYS','SYSTEM')
order  by OWNER, TABLE_NAME
;
